@@ -1,5 +1,7 @@
 from sklearn.preprocessing import StandardScaler
 from abc import ABC, abstractmethod
+from entmoot.space import Space
+from typing import Optional
 
 import numpy as np
 
@@ -39,7 +41,7 @@ class DistanceMetric(ABC):
 
         pass
 
-    def get_max_space_scaled_dist(self, ref_points, x_means, x_stddev, model):
+    def get_max_space_scaled_dist(self, ref_points, x_shifts, x_scalers, model):
         # computes maximum distance in search space
         lb = np.asarray(
             [model._cont_var_dict[i].lb for i in model._cont_var_dict.keys()]
@@ -48,8 +50,8 @@ class DistanceMetric(ABC):
             [model._cont_var_dict[i].ub for i in model._cont_var_dict.keys()]
         )
 
-        lb_std = np.divide(lb - x_means, x_stddev)
-        ub_std = np.divide(ub - x_means, x_stddev)
+        lb_std = np.divide(lb - x_shifts, x_scalers)
+        ub_std = np.divide(ub - x_shifts, x_scalers)
 
         max_dist = self.get_distance(
             lb_std,
@@ -99,7 +101,7 @@ class SquaredEuclidean(DistanceMetric):
         return dist
 
     def add_exploration_to_gurobi_model(self,
-        ref_points, x_means, x_stddev, distance_bound, model, add_rhs=None):
+        ref_points, x_shifts, x_scalers, dim_scaler, distance_bound, model, add_rhs=None):
         """Adds exploration constraints to a gurobi optimization model.
         Incentivizes solutions far away from reference points.
 
@@ -109,13 +111,14 @@ class SquaredEuclidean(DistanceMetric):
             Each row of `n_rows` is a reference point. Each dimension of `n_dim` 
             is the numerical value of a continuous variable.
 
-        x_means : np.array, shape (n_dims,)
-            Each dimension is the mean value of a continuous variable used to 
-            scale the data set.
+        x_shifts : np.array, shape (n_dims,)
+            Each dimension is the value by which the continuous variable is shifted.
 
-        x_stddev : np.array, shape (n_dims,)
-            Each dimension is the std value of a continuous variable used to 
-            scale the data set.
+        x_scalers : np.array, shape (n_dims,)
+            Each dimension is the value by which the continuous variable is scaled.
+
+        dim_scaler : int
+            Value by which the sum of uncertainties is scaled, i.e. either 1 or n_dims
 
         distance_bound : float
             Defines the maximum value that the exploration term can take.
@@ -139,7 +142,7 @@ class SquaredEuclidean(DistanceMetric):
                 vtype='C'
             )
 
-        def distance_ref_point_i(model, xi_ref, x_mean, x_stddev, add_rhs=None):
+        def distance_ref_point_i(model, xi_ref, x_shifts, x_scalers, dim_scaler, add_rhs=None):
             # function returns constraints capturing the standardized
             # exploration distance
             c_x = model._cont_var_dict
@@ -147,15 +150,15 @@ class SquaredEuclidean(DistanceMetric):
             n_features = len(xi_ref)
 
             diff_to_ref_point_i = quicksum(
-                ( (xi_ref[j] - (c_x[key]-x_mean[j]) / x_stddev[j]) * \
-                    (xi_ref[j] - (c_x[key]-x_mean[j]) / x_stddev[j]) )
+                ( (xi_ref[j] - (c_x[key]-x_shifts[j]) / x_scalers[j]) * \
+                    (xi_ref[j] - (c_x[key]-x_shifts[j]) / x_scalers[j]) )
                 for j,key in enumerate(model._cont_var_dict.keys())
             )
-            
+
             if add_rhs is None:
-                return alpha <= diff_to_ref_point_i
+                return dim_scaler*alpha <= diff_to_ref_point_i
             else:
-                return alpha <= diff_to_ref_point_i + add_rhs
+                return dim_scaler*alpha <= diff_to_ref_point_i + add_rhs
 
         # add exploration distances as quadratic constraints to the model
         for i in range(n_ref_points):
@@ -166,14 +169,14 @@ class SquaredEuclidean(DistanceMetric):
 
             model.addQConstr(
                 distance_ref_point_i(
-                    model, ref_points[i], x_means, x_stddev, temp_rhs
+                    model, ref_points[i], x_shifts, x_scalers, dim_scaler, temp_rhs
                 ),
                 name=f"std_const_{i}"
             )
         model.update()
 
     def add_penalty_to_gurobi_model(self,
-        ref_points, x_means, x_stddev, model, add_rhs=None):
+        ref_points, x_shifts, x_scalers, dim_scaler, model, add_rhs=None):
         """Adds penalty constraints to a gurobi optimization model.
         Incentivizes solutions close to reference points.
 
@@ -183,13 +186,14 @@ class SquaredEuclidean(DistanceMetric):
             Each row of n_rows is a reference point. Each dimension of n_dim is 
             the numerical value of a continuous variable.
 
-        x_means : np.array, shape (n_dims,)
-            Each dimension is the mean value of a continuous variable used to 
-            scale the data set.
+        x_shifts : np.array, shape (n_dims,)
+            Each dimension is the value by which the continuous variable is shifted.
 
-        x_stddev : np.array, shape (n_dims,)
-            Each dimension is the std value of a continuous variable used to 
-            scale the data set.
+        x_scalers : np.array, shape (n_dims,)
+            Each dimension is the value by which the continuous variable is scaled.
+
+        dim_scaler : int
+            Value by which the sum of uncertainties is scaled, i.e. either 1 or n_dims
 
         distance_bound : float
             Defines the maximum value that the exploration term can take.
@@ -205,7 +209,7 @@ class SquaredEuclidean(DistanceMetric):
 
         # big m is required to formulate the constraints
         model._big_m = \
-            self.get_max_space_scaled_dist(ref_points, x_means, x_stddev, model)
+            self.get_max_space_scaled_dist(ref_points, x_shifts, x_scalers, model)
         
         # binary variables b_ref correspond to active cluster centers
         model._b_ref = \
@@ -224,7 +228,7 @@ class SquaredEuclidean(DistanceMetric):
                 vtype='C'
             )
 
-        def distance_ref_point_k(model, xk_ref, k_ref, x_mean, x_stddev, add_rhs=None):
+        def distance_ref_point_k(model, xk_ref, k_ref, x_shifts, x_scalers, dim_scaler, add_rhs=None):
             # function returns constraints capturing the standardized
             # penalty distance
             c_x = model._cont_var_dict
@@ -233,15 +237,15 @@ class SquaredEuclidean(DistanceMetric):
             n_features = len(xk_ref)
 
             diff_to_ref_point_k = quicksum(
-                ( (xk_ref[j]  - (c_x[key]-x_mean[j]) / x_stddev[j]) * \
-                    (xk_ref[j]  - (c_x[key]-x_mean[j]) / x_stddev[j]) )
+                ( (xk_ref[j]  - (c_x[key]-x_shifts[j]) / x_scalers[j]) * \
+                    (xk_ref[j]  - (c_x[key]-x_shifts[j]) / x_scalers[j]) )
                 for j,key in enumerate(model._cont_var_dict.keys())
             )
             big_m_term = model._big_m*(1-b_ref[k_ref])
             if add_rhs is None:
-                return diff_to_ref_point_k <= alpha + big_m_term
+                return diff_to_ref_point_k <= dim_scaler*(alpha + big_m_term)
             else:
-                return diff_to_ref_point_k + add_rhs[k_ref] <= alpha + big_m_term
+                return diff_to_ref_point_k + add_rhs[k_ref] <= dim_scaler*(alpha + big_m_term)
 
         # add penalty distances as quadratic constraints to the model
         for k in range(n_ref_points):
@@ -250,8 +254,10 @@ class SquaredEuclidean(DistanceMetric):
                     model, 
                     ref_points[k], 
                     k, 
-                    x_means, 
-                    x_stddev
+                    x_shifts,
+                    x_scalers,
+                    dim_scaler,
+                    add_rhs
                 ),
                 name=f"std_const_{k}"
             )
@@ -310,7 +316,7 @@ class Manhattan(DistanceMetric):
         return dist
 
     def add_exploration_to_gurobi_model(self,
-        ref_points, x_means, x_stddev, distance_bound, model, add_rhs=None):
+        ref_points, x_shifts, x_scalers, dim_scaler, distance_bound, model, add_rhs=None):
         """Adds exploration constraints to a gurobi optimization model.
         Incentivizes solutions far away from reference points.
 
@@ -320,13 +326,14 @@ class Manhattan(DistanceMetric):
             Each row of `n_rows` is a reference point. Each dimension of `n_dim` 
             is the numerical value of a continuous variable.
 
-        x_means : np.array, shape (n_dims,)
-            Each dimension is the mean value of a continuous variable used to 
-            scale the data set.
+        x_shifts : np.array, shape (n_dims,)
+            Each dimension is the value by which the continuous variable is shifted.
 
-        x_stddev : np.array, shape (n_dims,)
-            Each dimension is the std value of a continuous variable used to 
-            scale the data set.
+        x_scalers : np.array, shape (n_dims,)
+            Each dimension is the value by which the continuous variable is scaled.
+
+        dim_scaler : int
+            Value by which the sum of uncertainties is scaled, i.e. either 1 or n_dims
 
         distance_bound : float
             Defines the maximum value that the exploration term can take.
@@ -359,14 +366,14 @@ class Manhattan(DistanceMetric):
                         name="alpha", vtype='C')
 
         def distance_ref_point_i_for_feat_j(
-            model, xi_ref, i_ref, feat_j, i_var, x_mean, x_stddev, add_rhs=None):
+            model, xi_ref, i_ref, feat_j, i_var, x_shifts, x_scalers, add_rhs=None):
             # function returns constraints capturing the standardized
             # exploration distance
             c_x = model._cont_var_dict
 
             diff_to_ref_point_i = \
-                ( xi_ref[feat_j] - (c_x[i_var]-x_mean[feat_j]) / \
-                    x_stddev[feat_j] ) 
+                ( xi_ref[feat_j] - (c_x[i_var]-x_shifts[feat_j]) / \
+                    x_scalers[feat_j] )
             return diff_to_ref_point_i == model._c_x_aux_pos[i_ref, i_var] - \
                 model._c_x_aux_neg[i_ref, i_var]
 
@@ -376,7 +383,7 @@ class Manhattan(DistanceMetric):
                 # _c_x_aux_pos and _c_x_aux_neg
                 model.addConstr(
                     distance_ref_point_i_for_feat_j(model, 
-                        ref_points[i_ref], i_ref, feat_j, key, x_means, x_stddev),
+                        ref_points[i_ref], i_ref, feat_j, key, x_shifts, x_scalers),
                     name=f"std_const_feat_{key}_{i_ref}"
                 )
                 # add sos constraints that allow only one of the +/- vars, 
@@ -391,7 +398,7 @@ class Manhattan(DistanceMetric):
             # add exploration distances as linear constraints to the model
             if add_rhs is None:
                 model.addConstr(
-                    model._alpha <= quicksum(
+                    dim_scaler*model._alpha <= quicksum(
                         (model._c_x_aux_pos[i_ref, j] + model._c_x_aux_neg[i_ref, j])
                         for j in cont_feat_ids
                     ),
@@ -399,7 +406,7 @@ class Manhattan(DistanceMetric):
                 )
             else:
                 model.addConstr(
-                    model._alpha <= quicksum(
+                    dim_scaler*model._alpha <= quicksum(
                         (model._c_x_aux_pos[i_ref, j] + model._c_x_aux_neg[i_ref, j])
                         for j in cont_feat_ids
                     ) + add_rhs[i_ref],
@@ -408,7 +415,7 @@ class Manhattan(DistanceMetric):
         model.update()
 
     def add_penalty_to_gurobi_model(self,
-        ref_points, x_means, x_stddev, model, add_rhs=None):
+        ref_points, x_shifts, x_scalers, dim_scaler, model, add_rhs=None):
         """Adds penalty constraints to a gurobi optimization model.
         Incentivizes solutions close to reference points.
 
@@ -418,13 +425,14 @@ class Manhattan(DistanceMetric):
             Each row of `n_rows` is a reference point. Each dimension of `n_dim` 
             is the numerical value of a continuous variable.
 
-        x_means : np.array, shape (n_dims,)
-            Each dimension is the mean value of a continuous variable used to 
-            scale the data set.
+        x_shifts : np.array, shape (n_dims,)
+            Each dimension is the value by which the continuous variable is shifted.
 
-        x_stddev : np.array, shape (n_dims,)
-            Each dimension is the std value of a continuous variable used to 
-            scale the data set.
+        x_scalers : np.array, shape (n_dims,)
+            Each dimension is the value by which the continuous variable is scaled.
+
+        dim_scaler : int
+            Value by which the sum of uncertainties is scaled, i.e. either 1 or n_dims
 
         distance_bound : float
             Defines the maximum value that the exploration term can take.
@@ -441,7 +449,7 @@ class Manhattan(DistanceMetric):
 
         # big m is required to formulate the constraints
         model._big_m = \
-            self.get_max_space_scaled_dist(ref_points, x_means, x_stddev, model)
+            self.get_max_space_scaled_dist(ref_points, x_shifts, x_scalers, model)
         
         # two sets of variables are used to capture positive and negative 
         # parts of manhattan distance
@@ -465,14 +473,14 @@ class Manhattan(DistanceMetric):
                         name="alpha", vtype='C')
 
         def distance_ref_point_i_for_feat_j(
-            model, xi_ref, i_ref, feat_j, i_var, x_mean, x_stddev, add_rhs=None):
+            model, xi_ref, i_ref, feat_j, i_var, x_shifts, x_scalers, add_rhs=None):
             # function returns constraints capturing the standardized
             # exploration distance
             c_x = model._cont_var_dict
 
             diff_to_ref_point_i = \
-                ( xi_ref[feat_j] - (c_x[i_var]-x_mean[feat_j]) / \
-                    x_stddev[feat_j] ) 
+                ( xi_ref[feat_j] - (c_x[i_var]-x_shifts[feat_j]) / \
+                    x_scalers[feat_j] )
             return diff_to_ref_point_i == model._c_x_aux_pos[i_ref, i_var] - \
                 model._c_x_aux_neg[i_ref, i_var]
 
@@ -482,7 +490,7 @@ class Manhattan(DistanceMetric):
                 # _c_x_aux_pos and _c_x_aux_neg
                 model.addConstr(
                     distance_ref_point_i_for_feat_j(
-                        model, ref_points[i_ref], i_ref, feat_j, key, x_means, x_stddev),
+                        model, ref_points[i_ref], i_ref, feat_j, key, x_shifts, x_scalers),
                     name=f"std_const_feat_{key}_{i_ref}"
                 )
                 # add sos constraints that allow only one of the +/- vars, 
@@ -500,7 +508,7 @@ class Manhattan(DistanceMetric):
                     quicksum(
                         (model._c_x_aux_pos[i_ref, j] + model._c_x_aux_neg[i_ref, j])
                         for j in cont_feat_ids
-                    ) <= model._alpha + model._big_m*(1-model._b_ref[i_ref]),
+                    ) <= dim_scaler*(model._alpha + model._big_m*(1-model._b_ref[i_ref])),
                     name=f"alpha_sum"
                 )
             else:
@@ -509,7 +517,7 @@ class Manhattan(DistanceMetric):
                         (model._c_x_aux_pos[i_ref, j] + model._c_x_aux_neg[i_ref, j])
                         for j in cont_feat_ids
                     )  + add_rhs[i_ref] <= \
-                        model._alpha + model._big_m*(1-model._b_ref[i_ref]),
+                        dim_scaler*(model._alpha + model._big_m*(1-model._b_ref[i_ref])),
                     name=f"alpha_sum"
                 )    
         model.update()
@@ -678,7 +686,8 @@ class OF(NonSimilarityMetric):
 
 
 class DistanceBasedStd(ABC):
-    """Define a distance-based standard estimator.
+    """
+    Define a distance-based standard estimator.
 
     A `DistanceBasedStd` object is used to quantify model uncertainty based 
     on distance to reference points, e.g. data points. The underlying assumption 
@@ -687,58 +696,52 @@ class DistanceBasedStd(ABC):
     Use this class as a template if you want to develop your own distance-based
     measure.
 
-    Parameters
-    ----------
-    metric : string
-        Metric used to compute distances, e.g. squared euclidean, manhattan
-
-    Attributes
-    ----------
-    Xi : list
-        Points at which objective has been evaluated.
-    yi : scalar
-        Values of objective at corresponding points in `Xi`.
-    cont_dist_metric : DistanceMetric
-        Object used to compute distances between continuous variables.
-    x_means : list
-        Mean of attribute `Xi`.
-    x_scaler : list
-        Scalers of attribute `Xi`.
-    Xi_standard : list
-        Standardized `Xi` array.
-    ref_points : list
-        Points to which the distance is computed to estimate model uncertainty.
-        Is different for all child classes.
+    :params space: Space, search space object that contains all vars
+    :params unc_scaling: str, type of scaling used for uncertainty
+        "standard": uses sample mean as shift and sample std as scaling
+        "normalize": normalizes vars by lower and upper bounds provided
+    :params dist_metric: str,
+        type of distance metric used for non-categorical vars, i.e.
+        currently supported ["squared_euclidean", "manhattan"]
+    :params cat_dist_metric: str,
+        type of dist. metric used for uncertainty of categorical vars, i.e. check:
+            {S. Boriah, V. Chandola, V. Kumar,
+            Similarity Measures for Categorical Data: A Comparative Evaluation,
+            2008, SIAM}
+        currently available options in ["overlap", "goodall4", "of"]
     """
     def __init__(self,
-        metric_cont='sq_euclidean',
-        metric_cat='overlap',
-        space=None):
+        space: Space,
+        unc_scaling: str = "standard",
+        dist_metric: str = 'squared_euclidean',
+        cat_dist_metric: str = 'overlap'):
+
+        self.std_type = 'distance'
+
+        self.unc_scaling = unc_scaling
 
         # set distance metric for cont variables
-        if metric_cont == 'sq_euclidean':
+        if dist_metric == 'squared_euclidean':
             from entmoot.learning.distance_based_std import SquaredEuclidean
-            self.cont_dist_metric = SquaredEuclidean()
+            self.dist_metric = SquaredEuclidean()
 
-        elif metric_cont == 'manhattan':
+        elif dist_metric == 'manhattan':
             from entmoot.learning.distance_based_std import Manhattan
-            self.cont_dist_metric = Manhattan()
+            self.dist_metric = Manhattan()
 
         self.space = space
 
         # set similarity metric for cat variables
-        if metric_cat == "overlap":
+        if cat_dist_metric == "overlap":
             from entmoot.learning.distance_based_std import Overlap
             self.cat_sim_metric = Overlap()
             
-        elif metric_cat == "goodall4":
+        elif cat_dist_metric == "goodall4":
             from entmoot.learning.distance_based_std import Goodall4
-            assert self.space is not None, "Space parameter is needed for goodall4."
             self.cat_sim_metric = Goodall4()
 
-        elif metric_cat == "of":
+        elif cat_dist_metric == "of":
             from entmoot.learning.distance_based_std import OF
-            assert self.space is not None, "Space parameter is needed for of."
             self.cat_sim_metric = OF()
 
     def get_x_cont_vals(self, xi):
@@ -764,7 +767,7 @@ class DistanceBasedStd(ABC):
         """
         pass
 
-    def update(self, Xi, yi, cat_column=[]):
+    def update(self, Xi, yi, cat_column=None):
         """Update available data points which is usually done after every
         iteration.
 
@@ -779,6 +782,8 @@ class DistanceBasedStd(ABC):
         -------
         -
         """
+        if cat_column is None:
+            cat_column = []
 
         self.cat_idx = cat_column
         
@@ -812,25 +817,43 @@ class DistanceBasedStd(ABC):
 
         self.n_features = Xi.shape[1]
 
-        # compute mean and scaler of data set
+
         if list(self.Xi_cont):
-            standard_scaler = StandardScaler()
-            projected_features = standard_scaler.fit_transform(self.Xi_cont)
-            self.x_means = standard_scaler.mean_
-            self.x_scalers = standard_scaler.scale_
 
-            # standardize dataset
-            self.Xi_cont_standard = self.standardize_with_Xi(self.Xi_cont)
+            # standardize points for comparison
+            if self.unc_scaling == "standard":
+                standard_scaler = StandardScaler()
+                projected_features = standard_scaler.fit_transform(self.Xi_cont)
+                self.x_shifts = standard_scaler.mean_
+                self.x_scalers = standard_scaler.scale_
+                self.dim_scaler = 1
 
-        # compute scale coefficient
-        y_scaler = np.std(self.yi)
-        self.std_scale_coef = abs(y_scaler / self.n_features)
+                # standardize dataset
+                self.Xi_cont_scaled = self.scale_with_Xi(self.Xi_cont)
+
+            # normalize points for comparison
+            elif self.unc_scaling == "normalize":
+                self.x_shifts = np.asarray(
+                    [dim.low
+                     for idx, dim in enumerate(self.space.dimensions)
+                     if idx not in self.cat_idx]
+                )
+
+                self.x_scalers = np.asarray(
+                    [dim.high - dim.low
+                     for idx, dim in enumerate(self.space.dimensions)
+                     if idx not in self.cat_idx]
+                )
+                self.dim_scaler = len(self.space.dimensions)
+
+                # standardize dataset
+                self.Xi_cont_scaled = self.scale_with_Xi(self.Xi_cont)
 
         # update cat_sim_metric with new data
         cat_dims = self.get_x_cat_vals(self.space.dimensions)
         self.cat_sim_metric.update(self.Xi_cat, cat_dims)
 
-    def standardize_with_Xi(self, X):
+    def scale_with_Xi(self, X):
         """Standardize given input `X` based on attribute `Xi`.
 
         Parameters
@@ -844,7 +867,7 @@ class DistanceBasedStd(ABC):
         x_standard : numpy array, shape (n_rows, n_dims)
             Standardized array of `X`
         """
-        x_standard = np.divide(X - self.x_means, self.x_scalers)
+        x_standard = np.divide(X - self.x_shifts, self.x_scalers)
         return x_standard
 
     def get_closest_point_distance(self, X):
@@ -868,12 +891,11 @@ class DistanceBasedStd(ABC):
             X_cont = self.get_x_cont_vals(X)
 
             # standardize cont variables
-            x_cont_standard = self.standardize_with_Xi(X_cont)  
-            x_cont_standard = np.asarray(x_cont_standard)
+            x_cont_scaled = np.asarray(self.scale_with_Xi(X_cont))
 
             # compute all distances for cont variables
             dist_cont = \
-                self.cont_dist_metric.get_distance(ref_points_cont,x_cont_standard)
+                self.dist_metric.get_distance(ref_points_cont, x_cont_scaled)
 
         # compute all distances for cat variables
         if self.Xi_cat:
@@ -889,7 +911,7 @@ class DistanceBasedStd(ABC):
 
         return np.min(dist_cont)
 
-    def predict(self, X, scaled=True):
+    def predict(self, X):
         """Predict standard estimate at location `X`.
 
         Parameters
@@ -909,10 +931,10 @@ class DistanceBasedStd(ABC):
             ref_distance = self.get_closest_point_distance(Xi)
             dist[row_res] = ref_distance
 
-        if scaled:
-            dist *= self.std_scale_coef
-
-        return dist
+        if self.unc_scaling == "standard":
+            return dist
+        elif self.unc_scaling == "normalize":
+            return dist / len(self.space.dimensions)
 
     @abstractmethod
     def add_to_gurobi_model(self,model):
@@ -950,42 +972,36 @@ class DistanceBasedExploration(DistanceBasedStd):
     function. Exploration refers to incentivizing distance to reference points
     leading to a negative contribution of the distance measure to the objective
     function.
-    
-    Parameters
-    ----------
-    metric : string
-        Metric used to compute distances, e.g. squared euclidean, manhattan
-    zeta : scalar
-        Coefficient determining how the distance measure is bounded
 
-    Attributes
-    ----------
-    Xi : list
-        Points at which objective has been evaluated.
-    yi : scalar
-        Values of objective at corresponding points in `Xi`.
-    cont_dist_metric : DistanceMetric
-        Object used to compute distances between continuous variables.
-    x_means : list
-        Mean of attribute `Xi`.
-    x_scaler : list
-        Scalers of attribute `Xi`.
-    Xi_standard : list
-        Standardized `Xi` array.
-    ref_points : list
-        `ref_points` standardized to which the distance measure is computed.
-    ref_points_unscaled : list
-        Unscaled `ref_points` to which the distance measure is computed.
-    distance_bound : scalar
-        Bound of exploration measure to prohibit over-exploration
+
+    :params space: Space, search space object that contains all vars
+    :params unc_scaling: str, type of scaling used for uncertainty
+        "standard": uses sample mean as shift and sample std as scaling
+        "normalize": normalizes vars by lower and upper bounds provided
+    :params dist_metric: str,
+        type of distance metric used for non-categorical vars, i.e.
+        currently supported ["squared_euclidean", "manhattan"]
+    :params cat_dist_metric: str,
+        type of dist. metric used for uncertainty of categorical vars, i.e. check:
+            {S. Boriah, V. Chandola, V. Kumar,
+            Similarity Measures for Categorical Data: A Comparative Evaluation,
+            2008, SIAM}
+        currently available options in ["overlap", "goodall4", "of"]
+    :params zeta: Optional[float],
+        coefficient determining how the distance measure is bounded,
+        i.e. bound = abs(zeta * np.var(yi))
     """
     def __init__(self,
-        metric_cont='sq_euclidean',
-        metric_cat='overlap',
-        space=None,
-        zeta=0.5):
+        space: Space,
+        unc_scaling: str = "standard",
+        dist_metric: str = 'squared_euclidean',
+        cat_dist_metric: str = 'overlap',
+        zeta: Optional[float] = 0.5):
 
-        super().__init__(metric_cont,metric_cat,space)
+        super().__init__(space,
+                         unc_scaling=unc_scaling,
+                         dist_metric=dist_metric,
+                         cat_dist_metric=cat_dist_metric)
         self.zeta = zeta
 
     def set_params(self,**kwargs):
@@ -1003,7 +1019,7 @@ class DistanceBasedExploration(DistanceBasedStd):
         zeta = kwargs.get("zeta", 0.5)
         self.zeta = zeta
 
-    def update(self, Xi, yi, cat_column=[]):
+    def update(self, Xi, yi, cat_column=None):
         """Update available data points which is usually done after every
         iteration.
 
@@ -1018,6 +1034,9 @@ class DistanceBasedExploration(DistanceBasedStd):
         -------
         -
         """
+        if cat_column is None:
+            cat_column = []
+
         super().update(Xi, yi, cat_column=cat_column)
 
         self.ref_points_unscaled = self.Xi_cont
@@ -1025,14 +1044,17 @@ class DistanceBasedExploration(DistanceBasedStd):
         #self.ref_points = self.Xi_standard
 
         if list(self.Xi_cont):
-            self.ref_points_cont = self.Xi_cont_standard
+            self.ref_points_cont = self.Xi_cont_scaled
         self.ref_points_cat = np.asarray(self.Xi_cat, dtype=int)
 
         # compute upper bound of uncertainty
-        y_var = np.var(yi)
-        self.distance_bound = abs(self.zeta*y_var)
+        if self.unc_scaling == "standard":
+            y_var = np.var(yi)
+            self.distance_bound = abs(self.zeta*y_var)
+        elif self.unc_scaling == "normalize":
+            self.distance_bound = 1.0
 
-    def predict(self, X, scaled=True):
+    def predict(self, X):
         """Predict standard estimate at location `X`. By default `dist` is
         bounded by attribute `distance_bound`.
 
@@ -1047,13 +1069,17 @@ class DistanceBasedExploration(DistanceBasedStd):
             Returns distances to closest `ref_point` for every point per row
             in `n_rows`.
         """
-        dist = super().predict(X, scaled=scaled)
+        dist = super().predict(X)
 
         # prediction has max out at `distance_bound`
         dist[dist > self.distance_bound] = self.distance_bound
-        return dist
 
-    def get_gurobi_obj(self, model, scaled=True):
+        if self.unc_scaling == "standard":
+            return dist
+        elif self.unc_scaling == "normalize":
+            return dist / len(self.space.dimensions)
+
+    def get_gurobi_obj(self, model):
         """Get contribution of standard estimator to gurobi model objective
         function.
 
@@ -1069,10 +1095,7 @@ class DistanceBasedExploration(DistanceBasedStd):
         """
         # negative contributation of alpha requires non-convex flag in gurobi.
         model.Params.NonConvex=2
-        if scaled:
-            return -self.std_scale_coef*model._alpha
-        else:
-            return -model._alpha
+        return -model._alpha
 
     def add_to_gurobi_model(self,model):
         """Add standard estimator to gurobi model. Model details are 
@@ -1087,52 +1110,49 @@ class DistanceBasedExploration(DistanceBasedStd):
             self.cat_idx, self.ref_points_cat, model
         )
 
-        self.cont_dist_metric.add_exploration_to_gurobi_model(
-            self.ref_points_cont, 
-            self.x_means, 
-            self.x_scalers, 
+        self.dist_metric.add_exploration_to_gurobi_model(
+            self.ref_points_cont,
+            self.x_shifts,
+            self.x_scalers,
+            self.dim_scaler,
             self.distance_bound, 
             model,
             add_rhs=cat_rhs
         )
 
 class DistanceBasedPenalty(DistanceBasedStd):
-    """Defines a child class based on `DistanceBasedStd`. Penalty 
+    """Defines a child class based on `DistanceBasedStd`. Penalty
     refers to how the distance measure contributes to the acquisition
     function. Penalty refers to penalizing distance to reference points
     leading to a positive contribution of the distance measure to the objective
     function.
-    
-    Parameters
-    ----------
-    metric : string
-        Metric used to compute distances, e.g. squared euclidean, manhattan
 
-    Attributes
-    ----------
-    Xi : list
-        Points at which objective has been evaluated.
-    yi : scalar
-        Values of objective at corresponding points in `Xi`.
-    cont_dist_metric : DistanceMetric
-        Object used to compute distances between continuous variables.
-    x_means : list
-        Mean of attribute `Xi`.
-    x_scaler : list
-        Scalers of attribute `Xi`.
-    Xi_standard : list
-        Standardized `Xi` array.
-    ref_points : list
-        `ref_points` reference to which the distance measure is computed
-    n_ref_points : scalar
-        length of `ref_points"""
+
+    :params space: Space, search space object that contains all vars
+    :params unc_scaling: str, type of scaling used for uncertainty
+        "standard": uses sample mean as shift and sample std as scaling
+        "normalize": normalizes vars by lower and upper bounds provided
+    :params dist_metric: str,
+        type of distance metric used for non-categorical vars, i.e.
+        currently supported ["squared_euclidean", "manhattan"]
+    :params cat_dist_metric: str,
+        type of dist. metric used for uncertainty of categorical vars, i.e. check:
+            {S. Boriah, V. Chandola, V. Kumar,
+            Similarity Measures for Categorical Data: A Comparative Evaluation,
+            2008, SIAM}
+        currently available options in ["overlap", "goodall4", "of"]
+    """
 
     def __init__(self,
-        metric_cont='sq_euclidean',
-        metric_cat='overlap',
-        space=None):
+        space: Space,
+        unc_scaling: str = "standard",
+        dist_metric: str = 'squared_euclidean',
+        cat_dist_metric: str = 'overlap'):
 
-        super().__init__(metric_cont,metric_cat,space)
+        super().__init__(space,
+                         unc_scaling=unc_scaling,
+                         dist_metric=dist_metric,
+                         cat_dist_metric=cat_dist_metric)
 
     def set_params(self,**kwargs):
         """Sets parameters related to distance-based standard estimator.
@@ -1148,7 +1168,7 @@ class DistanceBasedPenalty(DistanceBasedStd):
         """
         pass
 
-    def update(self, Xi, yi, cat_column=[]):
+    def update(self, Xi, yi, cat_column=None):
         """Update available data points which is usually done after every
         iteration.
 
@@ -1163,6 +1183,9 @@ class DistanceBasedPenalty(DistanceBasedStd):
         -------
         -
         """
+        if cat_column is None:
+            cat_column = []
+
         super().update(Xi, yi, cat_column=cat_column)
 
         self.ref_points_unscaled = self.Xi_cont
@@ -1170,11 +1193,11 @@ class DistanceBasedPenalty(DistanceBasedStd):
         #self.ref_points = self.Xi_standard
 
         if list(self.Xi_cont):
-            self.ref_points_cont = self.Xi_cont_standard
+            self.ref_points_cont = self.Xi_cont_scaled
         self.ref_points_cat = np.asarray(self.Xi_cat, dtype=int)
 
 
-    def predict(self, X, scaled=True):
+    def predict(self, X):
         """Predict standard estimate at location `X`. Sign of `dist` is negative
         because it contributes as a penalty.
 
@@ -1189,10 +1212,13 @@ class DistanceBasedPenalty(DistanceBasedStd):
             Returns distances to closest `ref_point` for every point per row
             in `n_rows`.
         """
-        dist = super().predict(X, scaled=scaled)
-        return -dist
+        dist = super().predict(X)
+        if self.unc_scaling == "standard":
+            return -dist
+        elif self.unc_scaling == "normalize":
+            return -dist / len(self.space.dimensions)
 
-    def get_gurobi_obj(self, model, scaled=True):
+    def get_gurobi_obj(self, model):
         """Get contribution of standard estimator to gurobi model objective
         function.
 
@@ -1206,10 +1232,7 @@ class DistanceBasedPenalty(DistanceBasedStd):
         alpha : gurobipy.Var,
             Model variable that takes the value of the uncertainty measure.
         """
-        if scaled:
-            return self.std_scale_coef*model._alpha
-        else:
-            return model._alpha
+        return model._alpha
 
     def add_to_gurobi_model(self,model):
         """Add standard estimator to gurobi model. Model details are 
@@ -1224,10 +1247,11 @@ class DistanceBasedPenalty(DistanceBasedStd):
             self.cat_idx, self.ref_points_cat, model
         )
 
-        self.cont_dist_metric.add_penalty_to_gurobi_model(
+        self.dist_metric.add_penalty_to_gurobi_model(
             self.ref_points_cont, 
-            self.x_means, 
-            self.x_scalers, 
+            self.x_shifts,
+            self.x_scalers,
+            self.dim_scaler,
             model,
             add_rhs=cat_rhs
         )
\ No newline at end of file
